﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;
using System.Data;
using Infragistics.Windows.DataPresenter;
using Infragistics.Windows;
using System.Diagnostics;

namespace HierarchicalCheckBoxAll
{
    /// <summary>
    /// Interaction logic for MainWindow.xaml
    /// </summary>
    public partial class MainWindow : Window
    {
        DataSet set;

        public MainWindow()
        {
            InitializeComponent();

            set = GenerateData();
            //RecordSelector r;r.Record.IsSelected
            xamDataGrid1.DataSource = set.Tables["Parent"].DefaultView;
        }

        private DataSet GenerateData()
        {
            var data = new DataSet("SimpleData");
            data.EnforceConstraints = false;

            DataTable parent = new DataTable("Parent");
            parent.Columns.Add("IsSelected", typeof(Boolean));
            parent.Columns.Add("ID", typeof(int));
            parent.Columns.Add("Age", typeof(int));
            parent.Columns.Add("Name", typeof(string));

            parent.PrimaryKey = new[] { parent.Columns[0] };
          
            data.Tables.Add(parent);

            for (int i = 1; i < 4; i++)
            {

                data.Tables["Parent"].Rows.Add(new object[] {false, i,i*10, "Parent" + i });
                data.Tables["Parent"].Rows.Add(new object[] { false, i, i * 10, "Parent" + i * 10 });
                data.Tables["Parent"].Rows.Add(new object[] { false, i, i * 10, "Parent" + i * 10 + i });
            }

            data.AcceptChanges();

            return data;
        }

        private void CheckBox_Loaded(object sender, RoutedEventArgs e)
        {
            DataRecordPresenter r = Utilities.GetAncestorFromType(sender as DependencyObject, typeof(DataRecordPresenter), false) as DataRecordPresenter;
            if (r != null && r.IsHeaderRecord && (r.Record as HeaderRecord).AttachedToRecord.ParentRecord != null)
            {
                HeaderRecord hRecord = r.Record as HeaderRecord;

                if (hRecord.AttachedToRecord.ParentRecord.HasChildren)
                {
                    if (hRecord.AttachedToRecord.ParentRecord is ExpandableFieldRecord)
                    {
                        foreach (Record rec in (hRecord.AttachedToRecord.ParentRecord as ExpandableFieldRecord).ChildRecords)
                        {
                            rec.Tag = sender;
                        }
                    }
                    else
                        foreach (Record rec in (hRecord.AttachedToRecord.ParentRecord as GroupByRecord).ChildRecords)
                        {
                            rec.Tag = sender;
                        }
                    
                }
            }
            else
            {
                foreach (DataRecord rec in r.Record.RecordManager.Current)
                {
                    rec.Tag = sender;
                }
            }
        }

        private void CheckBox_Checked(object sender, RoutedEventArgs e)
        {
            DataRecordPresenter r = Utilities.GetAncestorFromType(sender as DependencyObject, typeof(DataRecordPresenter), false) as DataRecordPresenter;

            SetSelection(r, true);
        }

        private void CheckBox_Unchecked(object sender, RoutedEventArgs e)
        {
            DataRecordPresenter r = Utilities.GetAncestorFromType(sender as DependencyObject, typeof(DataRecordPresenter), false) as DataRecordPresenter;

            SetSelection(r, false);
        }

        void SetSelection(DataRecordPresenter r, bool operation)
        {
            if (r != null && r.IsHeaderRecord && (r.Record as HeaderRecord).AttachedToRecord.ParentRecord != null)
            {
                HeaderRecord hRecord = r.Record as HeaderRecord;

                if (hRecord.AttachedToRecord.ParentRecord.HasChildren)
                {
                    if (hRecord.AttachedToRecord.ParentRecord is ExpandableFieldRecord)
                    {
                        foreach (DataRecord rec in (hRecord.AttachedToRecord.ParentRecord as ExpandableFieldRecord).ChildRecords)
                        {
                            rec.IsSelected = operation;
                            rec.Cells["IsSelected"].Value = operation;
                        }
                    }
                    else
                    foreach (DataRecord rec in (hRecord.AttachedToRecord.ParentRecord as GroupByRecord).ChildRecords)
                    {
                        rec.IsSelected = operation;
                        rec.Cells["IsSelected"].Value = operation;
                    }
                }
            }
            else
            {
                if (r.Record.RecordManager.Current is GroupByRecordCollection)
                {
                    foreach (GroupByRecord rec in (r.Record.RecordManager.Current as GroupByRecordCollection))
                    {
                        foreach (DataRecord dr in rec.ChildRecords)
                        {
                            dr.IsSelected = operation;
                            dr.Cells["IsSelected"].Value = operation;
                        }
                        
                    }
                }
                else
                foreach (DataRecord rec in r.Record.RecordManager.Current)
                {
                    rec.IsSelected = operation;
                    rec.Cells["IsSelected"].Value = operation;
                }
            }
        }

        private void Child_CheckBox_Checked(object sender, RoutedEventArgs e)
        {
            DataRecordPresenter r = Utilities.GetAncestorFromType(sender as DependencyObject, typeof(DataRecordPresenter), false) as DataRecordPresenter;

            if(AreAllRecordsSelectedOrDeselected(r.Record as DataRecord, true) && (r.Record.Tag as CheckBox).IsChecked == false)
            {
                (r.Record.Tag as CheckBox).IsChecked = true;
                r.DataRecord.Cells["IsSelected"].Value = true;
            }
        }

        private void Child_CheckBox_Unchecked(object sender, RoutedEventArgs e)
        {
            DataRecordPresenter r = Utilities.GetAncestorFromType(sender as DependencyObject, typeof(DataRecordPresenter), false) as DataRecordPresenter;

            if (AreAllRecordsSelectedOrDeselected(r.Record as DataRecord, false) && (r.Record.Tag as CheckBox).IsChecked == true)
            {
                (r.Record.Tag as CheckBox).IsChecked = false;
                r.DataRecord.Cells["IsSelected"].Value = false;
            }
        }

        public bool AreAllRecordsSelectedOrDeselected(DataRecord r, bool getAllSeleted)
        {
            bool areAllSelected = true;
            if (r.ParentRecord != null && r.ParentRecord is ExpandableFieldRecord)
            {
                
                    foreach (Record rec in (r.ParentRecord as ExpandableFieldRecord).ChildRecords)
                    {
                        if (getAllSeleted)
                        {
                            areAllSelected = areAllSelected && rec.IsSelected;
                        }
                        else
                        {
                            areAllSelected = areAllSelected && !rec.IsSelected;
                        }
                    }
                }
            else
            {
                foreach (Record rec in r.DataPresenter.Records)
                {
                    if (getAllSeleted)
                    {
                        areAllSelected = areAllSelected && rec.IsSelected;
                    }
                    else
                    {
                        areAllSelected = areAllSelected && !rec.IsSelected;
                    }
                }
            }
            return areAllSelected;
        }

    }
}
